#!/usr/bin/perl -w

# Generic installer for LSB developer kits.
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

our @install_pkgs = ();
our @remove_pkgs = ();

# Read the configuration
if (-e 'inst-config.pl') {
	require 'inst-config.pl';
}
else {
    die "Could not find configuration file 'inst-config.pl'.\n";
}

# Construct the list of packages to be removed
foreach (@install_pkgs) {
	die "Package '$_' is missing!\n" if (!-f $_);
	if (m/^(.*)-[0-9][^\-]*-[0-9]+\.[^\-]+\.rpm$/) {
		push(@remove_pkgs, $1);
	}
	else {
		die "Incorrect name of the RPM package: $_\n";
	}
}

# Now figure out what kind of system we're running
my $is_deb_os = (-f '/etc/debian_version');

if ($is_deb_os) {
	# Convert RPM package names into DEB ones
	map { s/_/-/g } @remove_pkgs;
}

my %remove_pkgs_unique = map {$_ => 1} @remove_pkgs;
my $uninstall_cmd = '';
my $install_cmd = '';

if ($is_deb_os) {
	print <<DATA;

This system appears to be a variant of Debian GNU/Linux, such as
Debian itself, Ubuntu, MEPIS, Xandros, Linspire, etc.

DATA
	if (system('which alien >/dev/null 2>&1') != 0) {
		print <<DATA;
However, the alien package (which is necessary to install LSB packages
on Debian-based systems) is not currently installed. This is likely a
more serious problem, in that alien is required for your system to be
LSB-compliant. You should install the "lsb" package, and try running
this script again.
DATA
		exit(3);
	}
	foreach my $pkg_name (keys(%remove_pkgs_unique)) {
		my $inst = `dpkg-query -W -f '\${Status}' $pkg_name 2>/dev/null`;
		if ($? or ($inst ne 'install ok installed')) {
			delete $remove_pkgs_unique{$pkg_name};
		}
	}
	@remove_pkgs = keys(%remove_pkgs_unique);
	if (scalar(@remove_pkgs) > 0) {
		$uninstall_cmd = 'dpkg -r '.join(' ', @remove_pkgs);
	}
	$install_cmd = 'alien -dck '.join(' ', @install_pkgs).' 2>/dev/null | sed -e \'s/ .*//\' | xargs dpkg -i ';
}
else {
	if (system('which rpm >/dev/null 2>&1') == 0) {
		print <<DATA;

This system appears to be a RPM-based distribution, such as those from
Red Hat, SuSE/Novell, Mandriva, Asianux, etc.

DATA
	foreach my $pkg_name (keys(%remove_pkgs_unique)) {
		if (system("rpm --quiet -q $pkg_name") != 0) {
			delete $remove_pkgs_unique{$pkg_name};
		}
	}
	@remove_pkgs = keys(%remove_pkgs_unique);
	if (scalar(@remove_pkgs) > 0) {
		$uninstall_cmd = 'rpm --quiet -e '.join(' ', @remove_pkgs);
	}
	$install_cmd = 'rpm --quiet --nodeps -i '.join(' ', @install_pkgs);
	}
	else {
		print <<DATA;
I cannot determine how to install LSB packages on your system. You
may have luck unpacking the packages in this directory manually, using
tools such as rpm2cpio, but there is no guarantee that the packages
will work for their intended purpose. If you think your system should
be able to install LSB packages, please contact the Linux Foundation.
DATA
		exit(3);
	}
}

# Now that we've printed what we think, check to make sure we're
# right.
print "Is this correct? [Y/n] ";
my $answer = <STDIN>;
$answer =~ s/[\r\n]//g;
exit(4) if (($answer !~ m/^y/i) && ($answer ne ''));
print "\n";

# Figure out how to get root if we need to.
my $su_cmd = '';
if ($> != 0) {
	print <<DATA;
In order to install these packages, you need administrator
privileges. You are currently running this script as an unprivileged
user.

DATA
	$su_cmd = 'su root -c';
	if (system('which sudo >/dev/null 2>&1') == 0) {
		print 'You have sudo available.  Should I use it instead of \'su -c\'? [Y/n] ';
		$answer = <STDIN>;
		$answer =~ s/[\r\n]//g;
		print "\n";
		$su_cmd = 'sudo /bin/sh -c' if (($answer =~ m/^y/i) || ($answer eq ''));
	}

	print <<DATA;
Using the command "$su_cmd" to gain root access. Please type the
appropriate password if prompted.

DATA
}
else {
	$su_cmd = '/bin/sh -c';
}

# Install the packages
if ($uninstall_cmd ne '') {
	$uninstall_cmd = "echo 'Deinstalling previously installed versions...'; $uninstall_cmd 2>>error-log.txt; echo ; ";
}
system("$su_cmd \"rm -f error-log.txt >/dev/null 2>&1; $uninstall_cmd echo 'Installing packages...'; $install_cmd 2>>error-log.txt\"");

my $err_msg = '';

if ($?) {
	$err_msg = "There were some problems with the package installation.\n";
}

if (open(ERRLOG, 'error-log.txt')) {
	while (my $line = <ERRLOG>) {
		if ($line !~ m/warning: .*: Header V3 DSA signature: NOKEY, key ID/) {
			$err_msg = "There were some problems with the package installation.\nCheck error-log.txt for more information.\n";
			last;
		}
	}
	close(ERRLOG);
}
else {
	if ((-f 'error-log.txt') && (-s 'error-log.txt' > 0)) {
		$err_msg .= "Failed to read error-log.txt file.\nIt may contain important information about installation progress.\n";
	}
}

print $err_msg;
